@@ -4,7 +4,7 @@ use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '0.42';
+our $VERSION = '0.43';
 
 our $DEBUG = 0;
 our $USE_SAFE = 1;
@@ -321,8 +321,9 @@ particular: the filter expressions). Don't use it if you don't trust
 your paths.
 
 Since v0.41 the filter expressions are secured using L<Safe.pm|Safe>
-to only allow basic Perl core ops. To unrestrict this to pre-v0.41 raw
-C<eval> behaviour you can set C<$Data::DPath::USE_SAFE> to False:
+to only allow basic Perl core ops. This provides more safety but is
+also significantly slower. To unrestrict this to pre-v0.41 raw C<eval>
+behaviour you can set C<$Data::DPath::USE_SAFE> to False:
 
   local $Data::DPath::USE_SAFE;
   # dpath '//CCC//*[ unsecure_perl_expression ]'
@@ -856,7 +857,7 @@ The public repository is hosted on github:
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2010 Steffen Schwigon.
+Copyright 2008-2011 Steffen Schwigon.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
